#include <gtk/gtk.h>

#include <Scintilla.h>
#include <SciLexer.h>
#define PLAT_GTK 1
#include <ScintillaWidget.h>

#define MARGIN_SCRIPT_FOLD_INDEX 1


static int exit_app(GtkWidget*w, GdkEventAny*e, gpointer p) {
	gtk_main_quit();
	return w||e||p||1;	// Avoid warnings
}

static void scintilla_notify(GtkWidget *sciWidget, gint ctrlID, struct SCNotification *notifyData, gpointer userData)
{
	int lineNr = scintilla_send_message(SCINTILLA(sciWidget), SCI_LINEFROMPOSITION, (uptr_t)notifyData->position, 0);


	switch (notifyData->margin) {
		case MARGIN_SCRIPT_FOLD_INDEX:
			scintilla_send_message(SCINTILLA(sciWidget), SCI_TOGGLEFOLD, lineNr, 0);
			break;
	};
}

int main(int argc, char **argv) {
	GtkWidget *app;
	GtkWidget *editor;
	ScintillaObject *sci;

	gtk_init(&argc, &argv);
	app = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	editor = scintilla_new();
	sci = SCINTILLA(editor);

	gtk_container_add(GTK_CONTAINER(app), editor);
	gtk_signal_connect(GTK_OBJECT(app), "delete_event",
	  GTK_SIGNAL_FUNC(exit_app), 0);

	scintilla_set_id(sci, 0);
	gtk_signal_connect(GTK_OBJECT(editor), "sci-notify", G_CALLBACK(scintilla_notify), editor);
	gtk_widget_set_usize(editor, 500, 300);

#define SSM(m, w, l) scintilla_send_message(sci, m, w, l)

	SSM(SCI_STYLECLEARALL, 0, 0);
	SSM(SCI_SETLEXER, SCLEX_VARAQ, 0);
	SSM(SCI_STYLESETFORE, SCE_VARAQ_COMMENT, 0x404040);
	SSM(SCI_STYLESETFORE, SCE_VARAQ_STRING, 0x00FF00);
	SSM(SCI_STYLESETFORE, SCE_VARAQ_NUMBER, 0x20A5DA);
	SSM(SCI_STYLESETFORE, SCE_VARAQ_KEYWORD, 0xFF0000);

	SSM(SCI_SETMARGINTYPEN, MARGIN_SCRIPT_FOLD_INDEX, SC_MARGIN_SYMBOL);
	SSM(SCI_SETMARGINMASKN, MARGIN_SCRIPT_FOLD_INDEX, SC_MASK_FOLDERS);
	SSM(SCI_SETMARGINWIDTHN, MARGIN_SCRIPT_FOLD_INDEX, 20);
	SSM(SCI_SETMARGINSENSITIVEN, MARGIN_SCRIPT_FOLD_INDEX, 1);

	SSM(SCI_MARKERDEFINE, SC_MARKNUM_FOLDER, SC_MARK_BOXPLUS);
	SSM(SCI_MARKERDEFINE, SC_MARKNUM_FOLDEROPEN, SC_MARK_BOXMINUS);
	SSM(SCI_MARKERDEFINE, SC_MARKNUM_FOLDEREND, SC_MARK_EMPTY);
	SSM(SCI_MARKERDEFINE, SC_MARKNUM_FOLDERMIDTAIL, SC_MARK_EMPTY);
	SSM(SCI_MARKERDEFINE, SC_MARKNUM_FOLDEROPENMID, SC_MARK_EMPTY);
	SSM(SCI_MARKERDEFINE, SC_MARKNUM_FOLDERSUB, SC_MARK_EMPTY);
	SSM(SCI_MARKERDEFINE, SC_MARKNUM_FOLDERTAIL, SC_MARK_EMPTY);

	SSM(SCI_SETFOLDFLAGS, SC_FOLDFLAG_LINEAFTER_CONTRACTED, 0);

	SSM(SCI_INSERTTEXT, 0, (sptr_t)
	 "(* 99 balmey vo' Beer Daq var'aq *)\n"
	 "(* Sum Rune Berge / klingon mu'mey Sum Tati *)\n"
	 "\n"
	 "~ balmey {\n"
	 "latlh 1 rap\'a\' \"\" tam\n"
	 "{ woD \"s\" } ghobe\'chugh\n"
	 "\" bal\" tam tlheghrar tam woD\n"
	 "\" vo\' beer\" tlheghrar\n"
	 "} pong\n"
	 "\n"
	 "~ chuH tlhegh {\n"
	 "latlh latlh bottles tlheghrar \" Daq reD\" tlheghrar cha\'\n"
	 "latlh latlh bottles tlheghrar cha\'\n"
	 "\"tlhap wa\' bIng je juS \'oH bIng\" cha\'\n"
	 "wa\'boqHa\' latlh latlh { \"ghobe\'\" } ghobe'chugh latlh balmey tlheghrar \" Daq reD\" tlheghrar cha\'\n"
	 "\"\" cha\'\n"
	 "latlh { chuH tlhegh } HIja\'chugh\n"
	 "} pong\n"
	 "\n"
	 "99 chuH tlhegh\n"
	);
	SSM(SCI_COLOURISE, 0, -1);

	gtk_widget_show_all(app);
	gtk_widget_grab_focus(GTK_WIDGET(editor));
	gtk_main();

	return 0;
}
